/*
	Copyright (c) 1998-2001 Macromedia, Inc.	All Rights Reserved

	You may utilize this source file to create and compile object code for use within products
	you may create. THIS CODE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, AND
	MACROMEDIA DISCLAIMS ALL IMPLIED WARRANTIES INCLUDING, BUT NOT LIMITED TO, MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.	IN NO EVENT WILL MACROMEDIA BE
	LIABLE TO YOU FOR ANY CONSEQUENTIAL, INDIRECT OR INCIDENTAL DAMAGES ARISING OUT OF YOUR
	USE OR INABILITY TO USE THIS CODE.
*/


#define INITGUID 1

#include "PrintXtra.h"

#include <string.h>
#include <stdlib.h>
#include "xclassver.h"
#include "moatry.h"

#include "driservc.h"
#include "drivalue.h"

#include "mmivalue.h"
#include "mmillist.h"
#include "mmiplist.h"
#include "mmidate.h"
#include "mmiclr.h"
#include "mmiservc.h"

#include <displays.h>
#include <folders.h>
#include <stat.h>
#include <Resources.h>
#include <MacWindows.h>
#include <carbon.h>

#define strechtopage 1

/*******************************************************************************
 * SCRIPTING XTRA MESSAGE TABLE DESCRIPTION.
 *
 * The general format is:
 * xtra <nameOfXtra>
 * new object me [ args ... ]
 * <otherHandlerDefintions>
 * --
 * The first line must give the name of the Scripting xtra.
 * The remaining lines give the names of the handlers that this xtra implements
 * along with the required types of the arguments for each handler.
 * 
 * -- Pre-defined handler new 
 * The new handler will be called when a child object is created, 
 * the first argument is always the child object and you defined any remaining arguments.
 * The new handler is the place to initialize any memory required by the child object.
 * 
 * -- Simple Handler Definitions
 * Each handler definition line is format like this:
 * <handlerName> <argType1> <argName1>, <argType2> <argName2> ...
 * The first word is the handler name. Following this are types description for
 * the argument to the handler, each separated by a comma.
 * The argument name <argName>, may be omited.
 * Permited argument types are: 
 * 	integer 
 * 	float
 * 	string
 * 	symbol
 * 	object
 * 	any
 * 	*
 * For integer, float, string, symbol, and object, the type of the argument must 
 * match. The type any means allow any type. The asterisk (*) means any number and 
 * any type of arguments.
 * 
 * The first argument is the child object and is always declared to be of type object.
 * 
 * -- Global Handlers
 * An asterisk (*) preceeding the handler name signifies a global handler.
 * This handler is at the global scope level and can be called from any
 * movie.
 * 
 * -- Xtra level handlers
 * A plus (+) preceeding the handler name signifies an Xtra level handler. 
 * This kind of handler can be called directly from the Xtra reference,
 * without creating a child object.
 * 
 * The enumerated list that follows must correspond directly with the msgTable 
 * (i.e. they must be in the same order).
 * 
 *******************************************************************************/ 
 
 /* This is the list of handlers for the xtra. The versionInfo string is combined
 /*	with the msgTable string in the register method to create a single string that
 /* used when registering the xtra as a scripting xtra. */

static char versionInfo[] = "xtra PrintXtra -- version %s.%s.%s\n";
static char msgTable[] = { 
	"new object me\n"  /* standard first handler entry in all message tables */
	//"Startdoc object me,integer dojob,integer orientation,string pathtofile --\n"
	"Startdoc object me,string docname,integer mode --start doc mode 0=PORTRAIT, 1=LANDSCAPE +16 do printdialog\n"
	"enddoc object me -- \n"
	"Startpage object me --start page \n"
	"Endpage object me--end page\n"
//	"printtext object me, * --\n"
//	"printscreen object me,*--\n"
	"Printtext object me,integer startx,integer starty,string textstring,integer wid,integer hite,integer mode,integer style --print text\n"
	"Printscreen object me,integer mode,integer sx,integer sy,integer ex,integer ey,integer px,integer py,integer pex,integer pey--print screen \n"

	"pagesetup object me,*--\n"
	"SetFont object me,string fontname--\n"
	};

/* 	This is the enumerated scripting method list. This list should
 *	directly correspond to the msgTable defined above. It is used
 *	to dispatch method calls via the methodSelector. The 'm_XXXX' method must
 *	be last.
 */

enum 
{
	m_new = 0, /* standard first entry */
	m_startdoc,
	m_enddoc,
	m_startpage,
	m_endpage,
	m_printtext,
	m_printscreen,
	m_pagesetup,
	m_setfont,
	m_XXXX
};

PMPrintSettings ps;
PMPageFormat	pf;
PMPrintContext	pc;
GrafPtr			oldport;
CGrafPtr		printport;
int				vres,hres;
Rect			pagerect1,lastrect;

float			scalex,scaley;
PMOrientation	orient;
PMPrintSession	sess;
PIMoaMmGC 		pMmGC1;
PIMoaDrPlayer	pIMoaDrPlayer;
short			fontnum;

#if TARGET_API_MAC_CARBON
PMRect			pagerect;
#else
Rect		pagerect;
THPrint		myprint;
TPrPort		*myport;
TPrStatus	prStatus;
#endif


/* ============================================================================= */
/* Xtra Glue Stuff */
/* ============================================================================= */

#define XTRA_VERSION_NUMBER XTRA_CLASS_VERSION

BEGIN_XTRA
	BEGIN_XTRA_DEFINES_CLASS(PrintXtra, XTRA_CLASS_VERSION)
		CLASS_DEFINES_INTERFACE(PrintXtra, IMoaRegister, XTRA_VERSION_NUMBER)
		CLASS_DEFINES_INTERFACE(PrintXtra, IMoaMmXScript, XTRA_VERSION_NUMBER)
		/*
		 * ---> insert additional interface(s) -->
		 */ 
	END_XTRA_DEFINES_CLASS
END_XTRA


/* ============================================================================= */
/* Create/Destroy for class PrintXtra */
/* ============================================================================= */


STDMETHODIMP_(MoaError) MoaCreate_PrintXtra (PrintXtra FAR * This)
{
moa_try
		
	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaMmValue, (PPMoaVoid) &This->pValueInterface));
	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaMmUtils2, (PPMoaVoid) &This->pMoaUtils));

moa_catch
moa_catch_end
moa_try_end
}

STDMETHODIMP_(void) MoaDestroy_PrintXtra(PrintXtra FAR * This)
{
moa_try

	if (This->pValueInterface != NULL) 
		ThrowErr (This->pValueInterface->Release());
		
	if (This->pMoaUtils != NULL) 
		ThrowErr (This->pMoaUtils->Release());

moa_catch
moa_catch_end
moa_try_end_void
}


/* ============================================================================= */
/* Methods in PrintXtra_IMoaRegister */
/* ============================================================================= */

/*****************************************************************************
 *  Data needed for Registering
 *	---------------------------
 *	Specific code needed to register different types of Xtras.  The skeleton
 *	source files include minimal implementations appropriate for each Xtra
 *	type.  Current necessary actions:
 *
 *	Scripting Xtra:				Add the Scripting Xtra Message Table
 *	Sprite Asset Xtra:			Nothing
 *	Tool Xtra:					Nothing
 *	Transition Asset Xtra		Nothing
 *
 *  ****optional: Register as Safe for Shockwave!
 *****************************************************************************/ 

STD_INTERFACE_CREATE_DESTROY(PrintXtra, IMoaRegister)

BEGIN_DEFINE_CLASS_INTERFACE(PrintXtra, IMoaRegister)
END_DEFINE_CLASS_INTERFACE

/* ----------------------------------------------------------------------------- */
STDMETHODIMP PrintXtra_IMoaRegister::Register(
	PIMoaCache pCache, 
	PIMoaXtraEntryDict pXtraDict
)
{	
moa_try
	PIMoaRegistryEntryDict		pReg;
	MoaBool						bItsSafe = TRUE;
	char versionStr[256];
	PMoaVoid pMemStr = NULL;

	/* Register the lingo xtra */
	ThrowErr(pCache->AddRegistryEntry(pXtraDict, &CLSID_PrintXtra, &IID_IMoaMmXScript, &pReg));

	/* Register the method table */
	sprintf(versionStr, versionInfo, VER_MAJORVERSION_STRING, VER_MINORVERSION_STRING, VER_BUGFIXVERSION_STRING);
	pMemStr = pObj->pCalloc->NRAlloc(sizeof(msgTable) + sizeof(versionStr));
	ThrowNull(pMemStr);

	strcpy((char *)pMemStr, versionStr);
	strcat((char *)pMemStr, msgTable);

	ThrowErr(pReg->Put(kMoaDrDictType_MessageTable, pMemStr, 0, kMoaDrDictKey_MessageTable));

	/* Mark xtra as safe for shockwave - but only if it IS safe ! */
	/*
	ThrowErr(pReg->Put(kMoaMmDictType_SafeForShockwave, &bItsSafe, sizeof(bItsSafe), kMoaMmDictKey_SafeForShockwave));
	*/

moa_catch
moa_catch_end
	if (pMemStr)
		pObj->pCalloc->NRFree(pMemStr);
moa_try_end
}



/* ============================================================================= */
/*  Methods in PrintXtra_IMoaMmXScript */
/* ============================================================================= */

BEGIN_DEFINE_CLASS_INTERFACE(PrintXtra, IMoaMmXScript)
END_DEFINE_CLASS_INTERFACE

//******************************************************************************
PrintXtra_IMoaMmXScript::PrintXtra_IMoaMmXScript(MoaError FAR * pError)
//------------------------------------------------------------------------------
{
	*pError = kMoaErr_NoErr;
}	

//******************************************************************************
PrintXtra_IMoaMmXScript::~PrintXtra_IMoaMmXScript()
//------------------------------------------------------------------------------
{
}


/* ----------------------------------------------------------------------------- */
STDMETHODIMP PrintXtra_IMoaMmXScript::Call (PMoaDrCallInfo callPtr)
{
moa_try
	switch	( callPtr->methodSelector ) 
	{
		case m_new:
			{
			}
			break;
					
		case m_startdoc:
			ThrowErr(XX_startdoc(callPtr));
			break;

		case m_enddoc:
			ThrowErr(XX_enddoc(callPtr));
			break;

		case m_startpage:
			ThrowErr(XX_startpage(callPtr));
			break;

		case m_endpage:
			ThrowErr(XX_endpage(callPtr));
			break;

		case m_printtext:
			ThrowErr(XX_printtext(callPtr));
			break;

		case m_printscreen:
			ThrowErr(XX_printscreen(callPtr));
			break;

		case m_pagesetup:
			ThrowErr(XX_pagesetup(callPtr));
			break;

		case m_setfont:
			ThrowErr(XX_setfont(callPtr));
			break;

	}
moa_catch
moa_catch_end
moa_try_end
}


/*****************************************************************************
 *  Private Methods
 *  -------------------
 *  Implementation of Private Class Methods
 /*
 *  This is the actual code for the defined methods. They are defined as 
 *  functions, and called from the switch statement in Call().
 *
 ****************************************************************************/


void p2c(char * str)
{
unsigned char i=str[0];
memcpy(str,&str[1],i);
str[i]=0;
}

void c2p(char * str)
{
unsigned char i=strlen(str);
memmove(&str[1],str,i);
str[0]=i;
}

MoaError PrintXtra_IMoaMmXScript::XX_startdoc(PMoaDrCallInfo pCall)
{

//startdoc object me,integer dojob,integer orientation,string pathtofile,integer len

MoaMmValue		value;
OSType			folder='asup';
char			tdata[2048];
char			foldername[2048];
short			vrefnum=-1000;
long			dirid=-1000;
long			fakedir;
OSErr			err=0;
short			resfileref;
bool			changed;

long			o;
Rect			r;
PMRect			pr;
MoaRect			tr;
PMResolution res;

PIMoaDrPlayer	pPlayer;
PIMoaDrMovie	pMovie;
PIMoaCallback	pMyCallback;
PIMoaMmGC 		pMmGC;
MoaMmNativeGCInfo	nativeGC;

float		xresscr,yresscr,xresprt,yresprt;
Rect		trect;

moa_try

	ThrowErr (pObj->pCallback->QueryInterface(&IID_IMoaDrPlayer,(PPMoaVoid)&pPlayer));
	ThrowErr (pPlayer->GetActiveMovie(&pMovie));

 	err =pMovie->GetStageBufferGC(&pMmGC); 
	err = pMmGC->GetNativeGCInfo(&nativeGC);
	WindowPtr macWindow = (WindowPtr)nativeGC.data.mac_window.windowPtr;

	pciGetArgByIndex( pCall,3, &value );
	ThrowErr (pObj->pValueInterface->ValueToInteger( &value,&o));
	if((o & 0xf)==0)
		orient=kPMPortrait;
	else
		orient=kPMLandscape;

	pciGetArgByIndex( pCall, 2, &value );
	ThrowErr (pObj->pValueInterface->ValueToString( &value,(char*)&foldername,2048));

	err=FindFolder (kOnAppropriateDisk,folder,true,&vrefnum,&dirid);

	sprintf((char*)&tdata,"%s","\p4Mation");
	err=DirCreate (vrefnum,dirid,(const unsigned char*)&tdata,&fakedir);
	if(err==noErr)
		{
		sprintf((char*)&tdata,"%s",foldername);
		c2p((char*)&tdata);
		err=DirCreate (vrefnum,fakedir,(const unsigned char*)&tdata,&fakedir);
		}
	else
		{
		sprintf((char*)&tdata,"%s%s",":4Mation:",foldername);
		c2p((char*)&tdata);
		err=DirCreate (vrefnum,dirid,(const unsigned char*)&tdata,&fakedir);
		}
	
	err=FindFolder (kOnAppropriateDisk,folder,true,&vrefnum,&dirid);
	sprintf((char*)&tdata,"%s%s",":4Mation:",foldername);
	c2p((char*)&tdata);
	HSetVol ((StringPtr)&tdata,vrefnum,dirid);
	
	HCreateResFile (0,0,"\pPageFormat.rsrc");
	resfileref=HOpenResFile (vrefnum,fakedir,"\pPageFormat.rsrc",fsCurPerm);

#if TARGET_API_MAC_CARBON

	Boolean acc;
	
	PMCreateSession (&sess);

	PMCreatePrintSettings (&ps);

	PMSessionDefaultPrintSettings (sess,ps);

	PMCreatePageFormat (&pf);

	PMSessionDefaultPageFormat (sess,pf);
 	
	PMSetOrientation (pf,orient,false);

	PMSessionValidatePageFormat (sess,pf,&changed);
	
	if((o & 0xf0)==0x10)
		PMSessionPrintDialog (sess,ps,pf,&acc);
 

	PMSessionBeginDocument (sess,ps,pf);

	PMSessionGetGraphicsContext (sess,0,(void **)&printport);

	CloseResFile (resfileref);
	
	PMGetAdjustedPageRect (pf,&pr);
	PMGetAdjustedPageRect (pf,&pagerect);

   	PMGetResolution (pf,&res);
 
	vres=res.vRes/2;
	hres=res.hRes/2;

	pagerect.top=pagerect.top+vres/2;
	pagerect.left=pagerect.left+hres/2;
	pagerect.bottom=pagerect.bottom-vres/2;
	pagerect.right=pagerect.right-hres/2;

 	MacToMoaRect(&pr, &tr);
 	
	GetPortBounds((CGrafPtr)macWindow, &trect);
	
	xresscr = (float) trect.right; 
    yresscr = (float) trect.bottom;

#else
	short	temp;	
	PrOpen();
	myprint=(THPrint) NewHandle (sizeof (TPrint));
	PrintDefault(myprint);
	PrValidate(myprint);

	if (orient==kPMLandscape)//1=port 2= landscape
		{
		temp=(**myprint).rPaper.bottom;
		(**myprint).rPaper.bottom=(**myprint).rPaper.right;
		(**myprint).rPaper.right=temp;
		(**myprint).prInfo.rPage.bottom=(**myprint).rPaper.bottom;
		(**myprint).prInfo.rPage.right=(**myprint).rPaper.right;
		(**myprint).prInfoPT.rPage.bottom=(**myprint).rPaper.bottom;
		(**myprint).prInfoPT.rPage.right=(**myprint).rPaper.right;
		(**myprint).prStl.wDev=(**myprint).prStl.wDev&0xfffd;
		PrValidate(myprint);
		}

		myport = PrOpenDoc(myprint,nil,nil);
		(**myprint).prJob.pFileName=(StringPtr)"\pTAT";
	
		vres=(**myprint).prInfo.iVRes/2;
		hres=(**myprint).prInfo.iHRes/2;

		pagerect=(**myprint).prInfo.rPage;
	
		pagerect.top=pagerect.top+vres/2;
		pagerect.left=pagerect.left+hres/2;
		pagerect.bottom=pagerect.bottom-vres/2;
		pagerect.right=pagerect.right-hres/2;

		xresscr = (float) (*macWindow).portRect.right-(*macWindow).portRect.left; 
		yresscr = (float) (*macWindow).portRect.bottom-(*macWindow).portRect.top;
 	
 		MacToMoaRect(&(**myprint).prInfo.rPage, &tr);

#endif

   	xresprt = (float) pagerect.right; 
    yresprt = (float) pagerect.bottom; 

    scalex = (xresprt / xresscr);  
    scaley = (yresprt / yresscr); 

	if (scalex>=scaley)
		scalex=scaley;
	if (scalex<scaley)
		scaley=scalex;
	
	if (pMmGC) 
	{
		pMmGC->Release();
		pMmGC = NULL;
	}
	if (pMovie) 
	{
		pMovie->Release();
		pMovie = NULL;
	}
	
	if (pPlayer) 
	{
		pPlayer->Release();
		pPlayer = NULL;
	}
		

	ThrowErr(pObj->pValueInterface->RectToValue(&tr, &pCall->resultValue));

moa_catch
moa_catch_end	
moa_try_end	

return kMoaErr_NoErr;
}

MoaError PrintXtra_IMoaMmXScript::XX_enddoc(PMoaDrCallInfo pCall)
{
moa_try
#if TARGET_API_MAC_CARBON

	PMSessionEndDocument (sess);
	PMRelease ((PMObject)pf);
	PMRelease ((PMObject)ps);
	PMRelease ((PMObject)sess);
#else
	PrCloseDoc(myport);
	
	if (((*myprint)->prJob.bJDocLoop = bSpoolLoop) && (!PrError() ) )
		PrPicFile(myprint, nil, nil, nil, &prStatus);
	PrClose();
	
	myport=nil;
	DisposeHandle ((Handle)myprint);
	myprint=nil;
#endif

moa_catch
moa_catch_end	
moa_try_end	
return kMoaErr_NoErr;
}

GrafPtr			pageport;

MoaError PrintXtra_IMoaMmXScript::XX_startpage(PMoaDrCallInfo pCall)
{

moa_try
#if TARGET_API_MAC_CARBON

	PMSessionBeginPage (sess,pf,NULL);
	PMSessionGetGraphicsContext (sess,0,(void **)&printport);
#else
	oldport=FrontWindow();
	PrOpenPage(myport,nil);
	SetPortWindowPort(&myport->gPort);
	PenNormal();
	SetPortWindowPort(oldport);

#endif

moa_catch
moa_catch_end	
moa_try_end
	
return kMoaErr_NoErr;
}

MoaError PrintXtra_IMoaMmXScript::XX_endpage(PMoaDrCallInfo pCall)
{

moa_try
#if TARGET_API_MAC_CARBON

	PMSessionEndPage (sess);
#else
	PrClosePage(myport);
#endif

moa_catch
moa_catch_end	
moa_try_end	


return kMoaErr_NoErr;
}

MoaError PrintXtra_IMoaMmXScript::XX_printtext(PMoaDrCallInfo pCall)
{
MoaMmValue		value;
Rect			tr;
PMRect			pr;
long			startx,starty,hite,wid,mode,xmode,ymode,ignore,style;
char			words[2048];
int				xres,yres;
int				txtwidth;
int				txthite;
FontInfo		finfo;
int				len;

moa_try

	pciGetArgByIndex( pCall,2, &value );
	ThrowErr (pObj->pValueInterface->ValueToInteger( &value,&startx));
	pciGetArgByIndex( pCall,3, &value );
	ThrowErr (pObj->pValueInterface->ValueToInteger( &value,&starty));

	pciGetArgByIndex( pCall,4, &value );
	ThrowErr (pObj->pValueInterface->ValueToString( &value,(char*)&words,2048));

	pciGetArgByIndex( pCall,5, &value );
	ThrowErr (pObj->pValueInterface->ValueToInteger( &value,&wid));
	pciGetArgByIndex( pCall,6, &value );
	ThrowErr (pObj->pValueInterface->ValueToInteger( &value,&hite));
	
	pciGetArgByIndex( pCall,7, &value );
	ThrowErr (pObj->pValueInterface->ValueToInteger( &value,&mode));
	pciGetArgByIndex( pCall,8, &value );
	ThrowErr (pObj->pValueInterface->ValueToInteger( &value,&style));

	len=strlen(words);
	
#if TARGET_API_MAC_CARBON

	PMSessionGetGraphicsContext (sess,0,(void **)&printport);
	SetPort(printport);
#else
	GetPort(&oldport);
	SetPort(&(*myport).gPort);
#endif

	TextFont(fontnum);
	TextFace(0);
	TextSize(hite);
	TextMode(srcOr);

	int temp;
	temp=style&0xf;
	switch (temp)
		{
		case 1:
			TextFace(italic);
			break;
			
		case 2:
			TextFace(underline);
		break;

		case 4:
			TextFace(outline); 
			break;
			
		default:
			TextFace(0);
		
		}

	style=style/16;
	if (style>0)
		TextFace(bold);
		

	xres=pagerect.right-pagerect.left;
	yres=pagerect.bottom-pagerect.top;

	txtwidth = TextWidth(&words,0,len);
	txtwidth=txtwidth+15;
	GetFontInfo(&finfo);
	txthite=finfo.ascent+finfo.descent+finfo.leading;
	

	xmode=mode&0xf;
	ymode=mode&0xf0;
	ignore=mode&0xf00;

	if (wid==0)
	{

	switch (xmode)
		{
		case 1:
			startx=(((xres/2)+hres/2) - (txtwidth / 2)); //centre just
			break;

		case 2:
			startx=pagerect.left;//left just
			break;

		case 3:
			startx=pagerect.right-txtwidth;//right just
			break;

		}

	switch (ymode)
		{
		case 16:
			starty=(txthite * starty)+pagerect.top;//line num
			break;

		case 32:
			starty=pagerect.bottom-txthite;//bottom just
			break;
		
		default:
			starty=starty;
		}

	Rect	sr1,temprect;
	bool	ret;

	SetRect(&sr1,startx,starty,startx+txtwidth,starty+txthite);
	
	ret=SectRect(&sr1,&lastrect,&temprect);
	lastrect=sr1;
	if (ignore==0x100)
		{
			if (ret)
				{
				starty=0;
				goto gexit;
				}
		}

	TETextBox((StringPtr)&words,len,&sr1,1);
	starty=lastrect.bottom;//-vres+1;
	}

	if (wid!=0)
		{
		long	res;
		Rect	tr;

		if (wid>0)
			{
			tr.left=startx;
			tr.top=starty;
			tr.bottom=hite;
			tr.right=(int)((float)((float)wid+(float)startx)*scalex);
			}
		else
			{
			int		xcentre,twid,thite,temp;
			
			wid=-wid;

			if (orient==kPMPortrait)
				{
				xcentre=pagerect.right-pagerect.left;
				temp=(float)xcentre/(float)wid;
				twid=xcentre-hres;
				thite=(int)((float)hite*temp);
				}
			else
				{
				xcentre=pagerect.bottom-pagerect.top;
				temp=(float)xcentre/(float)wid;
				twid=xcentre-hres;
				thite=(int)((float)hite*temp);
				}

			xcentre=pagerect.bottom-pagerect.top;

			tr.left=((xcentre/2)+hres)-(twid/2);
			tr.right=tr.left+twid;
			tr.top=starty+vres;
			tr.bottom=hite;

			div_t numlines;
			int	addlines;
			numlines=div(txtwidth,tr.right-tr.left);
			if (numlines.rem>0)
				addlines=numlines.quot+1;
			else
				addlines=numlines.quot;
				

			if (txtwidth>tr.right-tr.left)
				tr.bottom=tr.top+(addlines*txthite);
			else
				tr.bottom=tr.top+txthite;
			}

		switch (xmode)
			{
			case 1:
				res=tr.right-tr.left;
				tr.left=(((xres/2)+hres/2) - (res / 2));
				tr.right=tr.left+res;
				break;
			}

		TETextBox((StringPtr)&words,len,&tr,1);

		starty=tr.bottom;

		}

gexit:

	SetPort(oldport);
	ThrowErr(pObj->pValueInterface->IntegerToValue(starty, &pCall->resultValue));

moa_catch
moa_catch_end	
moa_try_end	

return kMoaErr_NoErr;
}

MoaError PrintXtra_IMoaMmXScript::XX_printscreen(PMoaDrCallInfo pCall)
{

MoaError			err = kMoaErr_NoErr;
MoaMmNativeGCInfo	nativeGC;
MoaMmValue  		value = {0};

Rect				srcrect,destrect,trect;
	
long				startx,starty,wid,hite,px,py,mode,pwid,phite;
float				xcentre,ycentre;
float				xresscr,yresscr,xresprt,yresprt;
int					model,modeh;
float				temp;
int					twid,thite;

PIMoaDrPlayer		pPlayer;
PIMoaDrMovie		pMovie;
PIMoaCallback		pMyCallback;
PIMoaMmGC 			pMmGC;

moa_try
	
#if TARGET_API_MAC_CARBON
	PMSessionGetGraphicsContext (sess,0,(void **)&printport);
#endif

	pciGetArgByIndex( pCall,2, &value );
	ThrowErr (pObj->pValueInterface->ValueToInteger( &value,&mode));
	pciGetArgByIndex( pCall,3, &value );
	ThrowErr (pObj->pValueInterface->ValueToInteger( &value,&startx));
	pciGetArgByIndex( pCall,4, &value );
	ThrowErr (pObj->pValueInterface->ValueToInteger( &value,&starty));
	pciGetArgByIndex( pCall,5, &value );
	ThrowErr (pObj->pValueInterface->ValueToInteger( &value,&wid));
	pciGetArgByIndex( pCall,6, &value );
	ThrowErr (pObj->pValueInterface->ValueToInteger( &value,&hite));
	pciGetArgByIndex( pCall,7, &value );
	ThrowErr (pObj->pValueInterface->ValueToInteger( &value,&px));
	pciGetArgByIndex( pCall,8, &value );
	ThrowErr (pObj->pValueInterface->ValueToInteger( &value,&py));

	ThrowErr (pObj->pCallback->QueryInterface(&IID_IMoaDrPlayer,(PPMoaVoid)&pPlayer));
	ThrowErr (pPlayer->GetActiveMovie(&pMovie));

 	err =pMovie->GetStageBufferGC(&pMmGC); 
	err = pMmGC->GetNativeGCInfo(&nativeGC);
	WindowPtr macWindow = (WindowPtr)nativeGC.data.mac_window.windowPtr;

#if TARGET_API_MAC_CARBON
	SetPort(printport);
#else
	SetPortWindowPort(&myport->gPort);
	//SetPort((GrafPort*)printport);
#endif

	PenNormal();

	model=0xf&mode;
	modeh=0xf0&mode;


	if (model==0)//do normal stretch
		{
		if (px==10000)
			{
			xcentre=pagerect.right-pagerect.left; 
			px=((xcentre/2)+hres)-(int((float(wid)*scalex)/2));//centre just
			}
		else
			px=int(float(px)*scalex);

		if (py==10000)
			{
			ycentre=pagerect.bottom-pagerect.top; 
			py=((ycentre/2)+vres)-(int((float(hite)*scaley)/2));//centre just
			}
		else
			py=int(float(py)*scaley);

		if (modeh==0)
			{
			SetRect(&srcrect,startx,starty,startx+wid,starty+hite);
			SetRect(&destrect,px,py,px+((int)((float)wid*scalex)),py+((int)((float)hite*scaley)));

			SetRect(&destrect,px,py,px+((int)((float)wid*scalex)),py+((int)((float)hite*scaley)));
			
#if TARGET_API_MAC_CARBON
			CopyBits(GetPortBitMapForCopyBits((CGrafPtr)macWindow),GetPortBitMapForCopyBits(printport),&srcrect,&destrect,srcCopy,nil);
#else
			CopyBits(&macWindow->portBits,&myport->gPort.portBits,&srcrect,&destrect,srcCopy,nil);
#endif
			}
		else
			{
			SetRect(&destrect,px,py,px+((int)((float)wid*scalex)),py+((int)((float)hite*scaley)));
			FrameRect(&destrect);
			}

		}

	if (model==1)//stretch to page
		{
		if (orient==kPMPortrait)
			{
			xcentre=pagerect.right-pagerect.left;
			temp=(float)xcentre/(float)wid;
			twid=xcentre-hres;
			thite=(int)((float)hite*temp);
			}
		else
			{
			xcentre=pagerect.bottom-pagerect.top;
			temp=(float)xcentre/(float)wid;
			twid=xcentre-hres;
			thite=(int)((float)hite*temp);
			}

		xcentre=pagerect.right-pagerect.left; 
		px=((xcentre/2)+hres/2)-(twid/2);//centre just

		if (py==10000)
			{
			ycentre=pagerect.bottom-pagerect.top; 
			py=((ycentre/2)+vres)-(int((float(thite))/2));//centre just
			}
		else
			py=int((float(py)*scaley)+vres);

		if (modeh==0)
			{
			SetRect(&srcrect,startx,starty,startx+wid,starty+hite);
			SetRect(&destrect,px,py,px+twid,py+thite);

#if TARGET_API_MAC_CARBON
			CopyBits(GetPortBitMapForCopyBits((CGrafPtr)macWindow),GetPortBitMapForCopyBits(printport),&srcrect,&destrect,srcCopy,nil);
#else
			CopyBits(&macWindow->portBits,&myport->gPort.portBits,&srcrect,&destrect,srcCopy,nil);
#endif
			}
		else
			{
			SetRect(&destrect,px,py,px+twid,py+thite);
			FrameRect(&destrect);
			}
		}

	if (model==2)//direct blit
		{
		pciGetArgByIndex( pCall,9, &value );
		ThrowErr (pObj->pValueInterface->ValueToInteger( &value,&pwid));
		pciGetArgByIndex( pCall,10, &value );
		ThrowErr (pObj->pValueInterface->ValueToInteger( &value,&phite));

		if (modeh==0)
			{
			SetRect(&srcrect,startx,starty,startx+wid,starty+hite);
			SetRect(&destrect,px,py,px+pwid,py+phite);
			SetRect(&destrect,px,py,px+pwid,py+phite);
			
			
#if TARGET_API_MAC_CARBON
			CopyBits(GetPortBitMapForCopyBits((CGrafPtr)macWindow),GetPortBitMapForCopyBits(printport),&srcrect,&destrect,srcCopy,nil);			
#else
			CopyBits(&macWindow->portBits,&myport->gPort.portBits,&srcrect,&destrect,srcCopy,nil);
#endif
			}
		if (modeh==16)
			{
			SetRect(&destrect,px,py,px+pwid,py+phite);
			FrameRect(&destrect);
			}
		py=destrect.bottom;//py+phite;
		goto exitg;
		}

	py=py+thite;
	py=(int)((float)py/scaley);
	py=destrect.bottom;


exitg:

#if TARGET_API_MAC_CARBON
	QDFlushPortBuffer( GetQDGlobalsThePort(), NULL );
#endif

	SetPortWindowPort(FrontWindow());
	if (pMmGC) 
	{
		pMmGC->Release();
		pMmGC = NULL;
	}
	if (pMovie) 
	{
		pMovie->Release();
		pMovie = NULL;
	}
	
	if (pPlayer) 
	{
		pPlayer->Release();
		pPlayer = NULL;
	}
		
	py=destrect.bottom;
	py=py-vres;

	ThrowErr(pObj->pValueInterface->IntegerToValue(py,&pCall->resultValue));

	return kMoaErr_NoErr;

moa_catch
moa_catch_end	
moa_try_end	

}

MoaError PrintXtra_IMoaMmXScript::XX_pagesetup(PMoaDrCallInfo pCall)
{
moa_try
moa_catch
moa_catch_end	
moa_try_end	
return kMoaErr_NoErr;
}

MoaError PrintXtra_IMoaMmXScript::XX_setfont(PMoaDrCallInfo pCall)
{
MoaError	err = kMoaErr_NoErr;
MoaMmValue	value = {0};
MoaChar		words[1024];
	
long		len;

moa_try

	pciGetArgByIndex( pCall,3, &value );
	ThrowErr (pObj->pValueInterface->ValueToInteger( &value,&len));

	pciGetArgByIndex( pCall,2, &value );
	ThrowErr (pObj->pValueInterface->ValueToString( &value,(char*)&words,len));

	GetFNum((StringPtr)&words,&fontnum);
	ThrowErr(pObj->pValueInterface->IntegerToValue(fontnum, &pCall->resultValue));

moa_catch
moa_catch_end	
moa_try_end	
return kMoaErr_NoErr;
}

